# Generated by Django 1.9.5 on 2016-06-05 19:24

import datetime

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("meal", "0001_fix161f"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Address",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "number",
                    models.PositiveIntegerField(
                        blank=True, null=True, verbose_name="street number"
                    ),
                ),
                ("street", models.CharField(max_length=100, verbose_name="street")),
                (
                    "apartment",
                    models.CharField(
                        blank=True, max_length=10, null=True, verbose_name="apartment"
                    ),
                ),
                (
                    "floor",
                    models.IntegerField(blank=True, null=True, verbose_name="floor"),
                ),
                ("city", models.CharField(max_length=50, verbose_name="city")),
                (
                    "postal_code",
                    models.CharField(max_length=6, verbose_name="postal code"),
                ),
            ],
            options={
                "verbose_name_plural": "addresses",
            },
        ),
        migrations.CreateModel(
            name="Client",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "billing_payment_type",
                    models.CharField(
                        choices=[
                            ("", "Payment type"),
                            ("check", "Check"),
                            ("cash", "Cash"),
                            ("debit", "Debit card"),
                            ("credit", "Credit card"),
                            ("eft", "EFT"),
                        ],
                        max_length=10,
                        null=True,
                        verbose_name="Payment Type",
                    ),
                ),
                (
                    "rate_type",
                    models.CharField(
                        choices=[
                            ("default", "Default"),
                            ("low income", "Low income"),
                            ("solidary", "Solidary"),
                        ],
                        default="default",
                        max_length=10,
                        verbose_name="rate type",
                    ),
                ),
                (
                    "emergency_contact_relationship",
                    models.CharField(
                        blank=True,
                        max_length=100,
                        null=True,
                        verbose_name="emergency contact relationship",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("D", "Pending"),
                            ("A", "Active"),
                            ("S", "Paused"),
                            ("N", "Stop: no contact"),
                            ("C", "Stop: contact"),
                            ("I", "Deceased"),
                        ],
                        default="D",
                        max_length=1,
                    ),
                ),
                (
                    "language",
                    models.CharField(
                        choices=[("en", "English"), ("fr", "French")],
                        default="fr",
                        max_length=2,
                    ),
                ),
                (
                    "alert",
                    models.TextField(
                        blank=True, null=True, verbose_name="alert client"
                    ),
                ),
                (
                    "delivery_type",
                    models.CharField(
                        choices=[("O", "Ongoing"), ("E", "Episodic")],
                        default="O",
                        max_length=1,
                    ),
                ),
                (
                    "gender",
                    models.CharField(
                        blank="True",
                        choices=[("", "Gender"), ("F", "Female"), ("M", "Male")],
                        default="U",
                        max_length=1,
                        null="True",
                    ),
                ),
                (
                    "birthdate",
                    models.DateField(
                        blank="True", default=django.utils.timezone.now, null="True"
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "clients",
            },
        ),
        migrations.CreateModel(
            name="Client_option",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "client",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="member.Client",
                        verbose_name="client",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Contact",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("Home phone", "Home phone"),
                            ("Cell phone", "Cell phone"),
                            ("Work phone", "Work phone"),
                            ("Email", "Email"),
                        ],
                        max_length=100,
                        verbose_name="contact type",
                    ),
                ),
                ("value", models.CharField(max_length=50, verbose_name="value")),
            ],
            options={
                "verbose_name_plural": "contacts",
            },
        ),
        migrations.CreateModel(
            name="Member",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "firstname",
                    models.CharField(max_length=50, verbose_name="firstname"),
                ),
                ("lastname", models.CharField(max_length=50, verbose_name="lastname")),
                (
                    "address",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="member.Address",
                        verbose_name="address",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "members",
            },
        ),
        migrations.CreateModel(
            name="Note",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("note", models.TextField(verbose_name="Note")),
                (
                    "date",
                    models.DateField(
                        default=django.utils.timezone.now, verbose_name="Date"
                    ),
                ),
                ("is_read", models.BooleanField(default=False, verbose_name="Is read")),
                (
                    "priority",
                    models.CharField(
                        choices=[("normal", "Normal"), ("urgent", "Urgent")],
                        default="normal",
                        max_length=15,
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="Notes",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Author",
                    ),
                ),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="Notes",
                        to="member.Member",
                        verbose_name="Member",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Notes",
            },
        ),
        migrations.CreateModel(
            name="Option",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=50, verbose_name="name")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="description"),
                ),
                (
                    "option_group",
                    models.CharField(
                        choices=[
                            ("side dish", "side dish"),
                            ("preparation", "Preparation"),
                            ("week delivery schedule", "Week delivery schedule"),
                        ],
                        max_length=100,
                        verbose_name="option group",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "options",
            },
        ),
        migrations.CreateModel(
            name="Referencing",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("referral_reason", models.TextField(verbose_name="Referral reason")),
                (
                    "work_information",
                    models.TextField(
                        blank=True, null=True, verbose_name="Work information"
                    ),
                ),
                (
                    "date",
                    models.DateField(
                        default=datetime.date.today, verbose_name="Referral date"
                    ),
                ),
                (
                    "client",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="client_referent",
                        to="member.Client",
                        verbose_name="client",
                    ),
                ),
                (
                    "referent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="member.Member",
                        verbose_name="referent",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "referents",
            },
        ),
        migrations.CreateModel(
            name="Restriction",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "client",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="member.Client",
                        verbose_name="client",
                    ),
                ),
                (
                    "restricted_item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="meal.Restricted_item",
                        verbose_name="restricted item",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="contact",
            name="member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="member_contact",
                to="member.Member",
                verbose_name="member",
            ),
        ),
        migrations.AddField(
            model_name="client_option",
            name="option",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="member.Option",
                verbose_name="option",
            ),
        ),
        migrations.AddField(
            model_name="client",
            name="billing_member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="member.Member",
                verbose_name="billing member",
            ),
        ),
        migrations.AddField(
            model_name="client",
            name="emergency_contact",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="emergency_contact",
                to="member.Member",
                verbose_name="emergency contact",
            ),
        ),
        migrations.AddField(
            model_name="client",
            name="member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Member",
                verbose_name="member",
            ),
        ),
    ]



from django.contrib import admin

from souschef.billing.models import Billing


# Register your models here.
class OrdersInline(admin.TabularInline):
    model = Billing.orders.through


class BillingAdmin(admin.ModelAdmin):
    inlines = [
        OrdersInline,
    ]


admin.site.register(Billing, BillingAdmin)



# Generated by Django 1.10 on 2016-09-12 19:23

import annoying.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("order", "0008_auto_20160912_1509"),
    ]

    operations = [
        migrations.CreateModel(
            name="Billing",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "total_amount",
                    models.DecimalField(
                        decimal_places=2, max_digits=6, verbose_name="total_amount"
                    ),
                ),
                ("billing_month", models.IntegerField()),
                ("billing_year", models.IntegerField()),
                ("created", models.DateTimeField(auto_now=True)),
                ("detail", annoying.fields.JSONField()),
                ("orders", models.ManyToManyField(to="order.Order")),
            ],
        ),
    ]



from datetime import (
    datetime,
    timedelta,
)

from django.contrib.admin.models import (
    ADDITION,
    LogEntry,
)
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext_lazy as _

from souschef.member.models import Client
from souschef.order.models import Order


class Command(BaseCommand):
    help = "Create new orders for a given delivery date for all\
            ongoing active clients using their meals defaults."

    def add_arguments(self, parser):
        parser.add_argument(
            "delivery_date",
            help="The date must be in the format YYYY-MM-DD",
        )
        parser.add_argument(
            "--days",
            help=(
                "The number of days to be created in advance."
                "There will not be duplicates."
            ),
            default=1,
            type=int,
        )

    def handle(self, *args, **options):
        start_date = datetime.strptime(options["delivery_date"], "%Y-%m-%d").date()
        days = options["days"]

        # Only active ongoing clients can receive orders
        clients = Client.ongoing.all()

        for i in range(days):
            # Create the orders
            delivery_date = start_date + timedelta(days=i)
            orders = Order.objects.auto_create_orders(delivery_date, clients)
            # Log the execution
            LogEntry.objects.log_action(
                user_id=1,
                content_type_id=1,
                object_id="",
                object_repr="Orders creation for "
                + str(delivery_date.strftime("%Y-%m-%d %H:%M")),
                action_flag=ADDITION,
            )
            print(
                _("{} orders created on {}: to be delivered on {}.").format(
                    len(orders), start_date, delivery_date
                )
            )



# Generated by Django 1.9.5 on 2016-06-05 20:09

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0001_fix161f"),
    ]

    operations = [
        migrations.AlterField(
            model_name="member",
            name="address",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="member.Address",
                verbose_name="address",
            ),
        ),
    ]



import csv

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import (
    LoginRequiredMixin,
    PermissionRequiredMixin,
)
from django.db import transaction
from django.db.models import (
    Case,
    IntegerField,
    Prefetch,
    Q,
    Sum,
    When,
)
from django.db.transaction import atomic
from django.http import (
    HttpResponse,
    HttpResponseRedirect,
    JsonResponse,
)
from django.shortcuts import get_object_or_404
from django.urls import (
    reverse,
    reverse_lazy,
)
from django.utils.decorators import classonlymethod
from django.utils.encoding import force_text
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.views import generic
from formtools.wizard.views import NamedUrlSessionWizardView

from souschef.delivery.views import calculateRoutePointsEuclidean
from souschef.meal.models import (
    COMPONENT_GROUP_CHOICES,
    COMPONENT_GROUP_CHOICES_SIDES,
)
from souschef.member.forms import (
    ClientAddressInformation,
    ClientBasicInformation,
    ClientPaymentInformation,
    ClientRestrictionsInformation,
    ClientScheduledStatusForm,
)
from souschef.member.formsets import (
    CreateRelationshipFormset,
    UpdateRelationshipFormset,
)
from souschef.member.models import (
    CELL,
    DAYS_OF_WEEK,
    EMAIL,
    HOME,
    WORK,
    Address,
    Client,
    Client_avoid_component,
    Client_avoid_ingredient,
    Client_option,
    ClientFilter,
    ClientScheduledStatus,
    ClientScheduledStatusFilter,
    Contact,
    DeliveryHistory,
    Member,
    Relationship,
    Restriction,
    Route,
)
from souschef.order.mixins import FormValidAjaxableResponseMixin
from souschef.order.models import (
    SIZE_CHOICES,
    Order,
)


class NamedUrlSessionWizardView_i18nURL(NamedUrlSessionWizardView):
    @classonlymethod
    def as_view(cls, *args, **kwargs):
        cls.i18n_url_names = kwargs.pop("i18n_url_names")
        return super().as_view(*args, **kwargs)

    def dispatch(self, request, *args, **kwargs):
        """
        Replace kwargs['step'] as non-i18n step name.
        """
        if "step" in kwargs:
            i18n_step = kwargs.pop("step")
            try:
                matched_tup = next(
                    tup
                    for tup in self.i18n_url_names
                    if force_text(tup[1]) == i18n_step
                )
                non_i18n_step = matched_tup[0]
            except StopIteration:
                # not found
                non_i18n_step = i18n_step
            finally:
                kwargs["step"] = non_i18n_step
        return super().dispatch(request, *args, **kwargs)

    def get_step_url(self, step):
        """
        Replace non-i18n step name as i18n step name.
        """
        non_i18n_step = step
        try:
            matched_tup = next(
                tup
                for tup in self.i18n_url_names
                if force_text(tup[0]) == non_i18n_step
            )
            i18n_step = matched_tup[1]
        except StopIteration:
            # not found
            i18n_step = non_i18n_step

        return super().get_step_url(i18n_step)


class ClientWizard(
    LoginRequiredMixin, PermissionRequiredMixin, NamedUrlSessionWizardView_i18nURL
):
    permission_required = "sous_chef.edit"
    template_name = "client/create/form.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context["weekday"] = DAYS_OF_WEEK
        context["meals"] = list(
            filter(
                lambda tup: tup[0] != COMPONENT_GROUP_CHOICES_SIDES,
                COMPONENT_GROUP_CHOICES,
            )
        )

        if "pk" in kwargs:
            context.update({"edit": True})
            context.update({"pk": kwargs["pk"]})

        form = context["form"]
        if isinstance(form, ClientBasicInformation):
            step_template = "client/partials/forms/basic_information.html"
        elif isinstance(form, ClientAddressInformation):
            step_template = "client/partials/forms/address_information.html"
        elif isinstance(form, ClientPaymentInformation):
            step_template = "client/partials/forms/payment_information.html"
        elif isinstance(form, ClientRestrictionsInformation):
            step_template = "client/partials/forms/dietary_restriction.html"
        elif isinstance(form, CreateRelationshipFormset):
            step_template = "client/partials/forms/relationships.html"
        else:
            step_template = None
        context["step_template"] = step_template

        return context

    def get_form_initial(self, step):
        """
        Load initial data.
        """
        initial = {}
        if "pk" in self.kwargs:
            pk = self.kwargs["pk"]
            client = Client.objects.get(id=pk)
            initial = self.load_initial_data(step, client)

        return self.initial_dict.get(step, initial)

    def done(self, form_list, form_dict, **kwargs):
        """
        Process the submitted and validated form data.
        """
        # Use form_dict which allows us to access the wizard’s forms
        # based on their step names.
        pk = None
        self.form_dict = form_dict

        if "pk" in kwargs:
            pk = kwargs["pk"]
        self.save(pk)
        messages.add_message(
            self.request, messages.SUCCESS, _("The client has been created")
        )
        return HttpResponseRedirect(reverse_lazy("member:list"))

    def load_initial_data(self, step, client):
        """
        Load initial for the given step and client.
        """
        initial = {
            "firstname": client.member.firstname,
            "lastname": client.member.lastname,
            "alert": client.alert,
            "gender": client.gender,
            "language": client.language,
            "birthdate": client.birthdate,
            "contact_value": client.member.home_phone,
            "street": client.member.address.street,
            "city": client.member.address.city,
            "apartment": client.member.address.apartment,
            "postal_code": client.member.address.postal_code,
            "delivery_note": client.delivery_note,
            "route": client.route,
            "latitude": client.member.address.latitude,
            "longitude": client.member.address.longitude,
            "distance": client.member.address.distance,
            "member": client.id,
            "same_as_client": True,
            "facturation": "",
            "billing_payment_type": "",
        }
        return initial

    def save_json(self, dictonary):
        json = {}

        for days, _Days in DAYS_OF_WEEK:
            json[f"size_{days}"] = dictonary.get(f"size_{days}")

            if json[f"size_{days}"] == "":
                json[f"size_{days}"] = None

            for meal, _Meals in COMPONENT_GROUP_CHOICES:
                if meal is COMPONENT_GROUP_CHOICES_SIDES:
                    continue  # skip "Sides"
                json[f"{meal}_{days}_quantity"] = dictonary.get(
                    f"{meal}_{days}_quantity"
                )

        return json

    @atomic
    def save(self, id=None):
        """
        Update or create the member and all its related data.
        """
        basic_information = self.form_dict["basic_information"].cleaned_data
        address_information = self.form_dict["address_information"].cleaned_data
        payment_information = self.form_dict["payment_information"].cleaned_data
        dietary_restriction = self.form_dict["dietary_restriction"].cleaned_data

        member, created = Member.objects.update_or_create(
            id=id,
            defaults={
                "firstname": basic_information.get("firstname"),
                "lastname": basic_information.get("lastname"),
            },
        )

        address, created = Address.objects.update_or_create(
            id=None if member.address is None else member.address.id,
            defaults={
                "street": address_information.get("street"),
                "apartment": address_information.get("apartment"),
                "city": address_information.get("city"),
                "postal_code": address_information.get("postal_code"),
                "latitude": address_information.get("latitude"),
                "longitude": address_information.get("longitude"),
                "distance": address_information.get("distance"),
            },
        )
        member.address = address
        member.save()

        member.add_contact_information(HOME, basic_information.get("home_phone"))
        member.add_contact_information(CELL, basic_information.get("cell_phone"))
        member.add_contact_information(EMAIL, basic_information.get("email"))

        billing_member = self.save_billing_member(member)

        client, created = Client.objects.update_or_create(
            id=member.id,
            defaults={
                "member": member,
                "language": basic_information.get("language"),
                "gender": basic_information.get("gender"),
                "birthdate": basic_information.get("birthdate"),
                "alert": basic_information.get("alert"),
                "rate_type": payment_information.get("facturation"),
                "billing_payment_type": payment_information.get("billing_payment_type"),
                "billing_member": billing_member,
                "delivery_type": dietary_restriction.get("delivery_type"),
                "meal_default_week": self.save_json(dietary_restriction),
                "route": address_information.get("route"),
                "delivery_note": address_information.get("delivery_note"),
                "status": "A" if dietary_restriction["status"] else "D",
            },
        )

        self.save_relationships(billing_member, client)
        self.save_preferences(client)

    def save_billing_member(self, member):
        payment_information = self.form_dict["payment_information"].cleaned_data

        if payment_information.get("same_as_client"):
            billing_member = member

        else:
            e_b_member = payment_information.get("member")
            if self.billing_member_is_member():
                billing_member = member
            elif e_b_member:
                e_b_member_id = (
                    e_b_member.split(" ")[0].replace("[", "").replace("]", "")
                )
                billing_member = Member.objects.get(pk=e_b_member_id)
            else:
                billing_address = Address.objects.create(
                    number=payment_information.get("number"),
                    street=payment_information.get("street"),
                    apartment=payment_information.get("apartment"),
                    floor=payment_information.get("floor"),
                    city=payment_information.get("city"),
                    postal_code=payment_information.get("postal_code"),
                )
                billing_address.save()

                billing_member = Member.objects.create(
                    firstname=payment_information.get("firstname"),
                    lastname=payment_information.get("lastname"),
                    address=billing_address,
                )
                billing_member.save()

        return billing_member

    def save_relationships(self, billing_member, client):
        relationships = self.form_dict["relationships"]
        results = []
        for relationship in relationships:
            # Avoid empty forms, at least one relationship form is
            # required by django formset validation. If we have
            # one form good filled and several empty forms, the 'is_valid'
            # method return True
            if relationship.changed_data:
                data = relationship.cleaned_data
                relationship_member = data.get("member")
                if self.billing_member_is_relationship(relationship, billing_member):
                    member = billing_member
                elif relationship_member:
                    relationship_member_id = (
                        relationship_member.split(" ")[0]
                        .replace("[", "")
                        .replace("]", "")
                    )
                    member = Member.objects.get(pk=relationship_member_id)
                else:
                    member = Member.objects.create(
                        firstname=data.get("firstname"), lastname=data.get("lastname")
                    )
                    if relationship.has_referent_relationship:
                        member.work_information = data.get("work_information")
                    member.save()
                    r_email = data.get("email", None)
                    r_work_phone = data.get("work_phone", None)
                    r_cell_phone = data.get("cell_phone", None)
                    if r_email:
                        member.add_contact_information(EMAIL, r_email)
                    if r_work_phone:
                        member.add_contact_information(WORK, r_work_phone)
                    if r_cell_phone:
                        member.add_contact_information(CELL, r_cell_phone)

                extra_fields = {}
                if relationship.has_referent_relationship:
                    extra_fields["referral_date"] = data.get("referral_date")
                    extra_fields["referral_reason"] = data.get("referral_reason")

                results.append(
                    Relationship.objects.create(
                        client=client,
                        member=member,
                        nature=data.get("nature"),
                        type=data.get("type"),
                        remark=data.get("remark"),
                        extra_fields=extra_fields,
                    )
                )

        return results

    def save_preferences(self, client):
        preferences = self.form_dict["dietary_restriction"].cleaned_data

        # Save meals schedule as a Client option
        client.set_simple_meals_schedule(preferences.get("meals_schedule"))

        # Save restricted items
        for restricted_item in preferences.get("restrictions"):
            Restriction.objects.create(client=client, restricted_item=restricted_item)

        # Save food preparation
        for food_preparation in preferences.get("food_preparation"):
            Client_option.objects.create(client=client, option=food_preparation)

        # Save ingredients to avoid
        for ingredient_to_avoid in preferences.get("ingredient_to_avoid"):
            Client_avoid_ingredient.objects.create(
                client=client, ingredient=ingredient_to_avoid
            )

        # Save components to avoid
        for component_to_avoid in preferences.get("dish_to_avoid"):
            Client_avoid_component.objects.create(
                client=client, component=component_to_avoid
            )

    def billing_member_is_member(self):
        basic_information = self.form_dict["basic_information"]
        payment_information = self.form_dict["payment_information"]

        b_firstname = basic_information.cleaned_data.get("firstname")
        b_lastname = basic_information.cleaned_data.get("lastname")

        p_firstname = payment_information.cleaned_data.get("firstname")
        p_lastname = payment_information.cleaned_data.get("lastname")

        if b_firstname == p_firstname and b_lastname == p_lastname:
            return True
        return False

    def billing_member_is_relationship(self, relationship, billing_member):
        r_firstname = relationship.cleaned_data.get("firstname")
        r_lastname = relationship.cleaned_data.get("lastname")

        if (
            r_firstname == billing_member.firstname
            and r_lastname == billing_member.lastname
        ):
            return True

        return False


class ClientList(LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    # Display the list of clients
    context_object_name = "clients"
    model = Client
    paginate_by = 20
    permission_required = "sous_chef.read"
    template_name = "client/list.html"

    def get_queryset(self):
        uf = ClientFilter(self.request.GET)
        return uf.qs.select_related("member", "route").prefetch_related(
            "member__member_contact"
        )

        # The queryset must be client

    def get_context_data(self, **kwargs):
        uf = ClientFilter(self.request.GET, queryset=self.get_queryset())

        context = super().get_context_data(**kwargs)

        # Here you add some variable of context to display on template
        context["filter"] = uf
        context["display"] = self.request.GET.get("display", "block")
        text = ""
        count = 0
        for getVariable in self.request.GET:
            if getVariable == "display" or getVariable == "page":
                continue
            for getValue in self.request.GET.getlist(getVariable):
                if count == 0:
                    text += "?" + getVariable + "=" + getValue
                else:
                    text += "&" + getVariable + "=" + getValue
                count += 1

        text = text + "?" if count == 0 else text + "&"
        context["get"] = text

        return context

    def get(self, request, **kwargs):
        self.format = request.GET.get("format", False)

        if self.format == "csv":
            return export_csv(self, self.get_queryset())

        return super().get(request, **kwargs)


DAYS = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]


def _get_csv_header():
    header = [
        "ID",
        "Client Firstname",
        "Client Lastname",
        "Client Status",
        "Client Alert",
        "Client Language",
        "Client Gender",
        "Client Birthdate",
        "Client Delivery",
        "Client Home Phone",
        "Client Cell Phone",
        "Client Work Phone",
        "Client Email",
        "Client Street",
        "Client Apartment",
        "Client City",
        "Client Postal Code",
        "Client Route",
        "Client Billing Type",
        "Billing Member",
        "Relationships",
        "Meal Default",
    ]

    header.extend(DAYS)

    return header


def _get_csv_row(obj, route):
    mealdefweek = obj.meal_default_week
    row = [
        obj.id,
        obj.member.firstname,
        obj.member.lastname,
        obj.get_status_display(),
        obj.alert,
        obj.language,
        obj.gender,
        obj.birthdate,
        obj.delivery_type,
        obj.member.home_phone,
        obj.member.cell_phone,
        obj.member.work_phone,
        obj.member.email,
        obj.member.address.street,
        obj.member.address.apartment,
        obj.member.address.city,
        obj.member.address.postal_code,
        route,
        obj.billing_payment_type,
        obj.billing_member,
        ", ".join(str(c) for c in obj.relationship_set.all()),
        mealdefweek,
    ]

    simple_meals_schedule = set(obj.simple_meals_schedule or [])
    for day in DAYS:
        row.append("x" if day.lower() in simple_meals_schedule else "")

    return row


def export_csv(self, queryset):
    response = HttpResponse(content_type="text/csv")
    response["Content-Disposition"] = "attachment; filename=client_export.csv"
    writer = csv.writer(response, csv.excel)
    writer.writerow(_get_csv_header())

    for obj in queryset:
        route = "" if obj.route is None else obj.route.name

        writer.writerow(_get_csv_row(obj, route))

    return response


class ClientView(LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView):
    # Display detail of one client
    model = Client
    queryset = Client.objects.prefetch_related(
        Prefetch(
            "scheduled_statuses",
            queryset=ClientScheduledStatus.objects.filter(
                operation_status__in=[
                    ClientScheduledStatus.TOBEPROCESSED,
                    ClientScheduledStatus.ERROR,
                ]
            ).order_by("change_date"),
            to_attr="unprocessed_scheduled_statuses",
        )
    )
    permission_required = "sous_chef.read"


class ClientInfoView(ClientView):
    template_name = "client/view/information.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["active_tab"] = "information"
        context["client_status"] = Client.CLIENT_STATUS
        return context


class ClientAddressView(ClientView):
    template_name = "client/view/address.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context


class ClientPaymentView(ClientView):
    template_name = "client/view/payment.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["active_tab"] = "billing"
        context["client_status"] = Client.CLIENT_STATUS
        return context


class ClientAllergiesView(ClientView):
    template_name = "client/view/allergies.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["active_tab"] = "prefs"
        context["client_status"] = Client.CLIENT_STATUS
        context["weekdays"] = DAYS_OF_WEEK
        sms = self.object.simple_meals_schedule
        if sms:
            weekdays_dict = dict(DAYS_OF_WEEK)
            context["delivery_days"] = list(map(lambda d: (d, weekdays_dict[d]), sms))

        context["components"] = list(
            filter(
                lambda t: t[0] != COMPONENT_GROUP_CHOICES_SIDES, COMPONENT_GROUP_CHOICES
            )
        )
        context["meals_default"] = dict(self.object.meals_default)
        context["size_choices"] = dict(SIZE_CHOICES)
        return context


class ClientStatusView(ClientView):
    template_name = "client/view/status.html"

    def get_default_ops_value(self):
        return self.request.GET.get("operation_status", None)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["active_tab"] = "status"
        context["client_status"] = Client.CLIENT_STATUS
        context["filter"] = ClientScheduledStatusFilter(
            {"operation_status": self.get_default_ops_value()},
            queryset=self.object.scheduled_statuses,
        )
        context["client_statuses"] = context["filter"].qs

        return context


class ClientOrderList(ClientView):
    template_name = "client/view/orders.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["orders"] = self.object.orders.prefetch_related("orders")
        context["client_status"] = Client.CLIENT_STATUS
        context["active_tab"] = "orders"
        context["show_actions_column"] = True
        return context


class ClientUpdateInformation(
    LoginRequiredMixin, PermissionRequiredMixin, generic.edit.FormView
):
    permission_required = "sous_chef.edit"
    template_name = "client/update/steps.html"

    def get_initial(self):
        client = get_object_or_404(
            Client.objects.select_related("route", "member__address").prefetch_related(
                "member__member_contact"
            ),
            pk=self.kwargs.get("pk"),
        )
        initial = {
            "firstname": client.member.firstname,
            "lastname": client.member.lastname,
            "alert": client.alert,
            "gender": client.gender,
            "language": client.language,
            "birthdate": client.birthdate,
            "home_phone": client.member.home_phone,
            "cell_phone": client.member.cell_phone,
            "email": client.member.email,
            "street": client.member.address.street,
            "city": client.member.address.city,
            "apartment": client.member.address.apartment,
            "postal_code": client.member.address.postal_code,
            "delivery_note": client.delivery_note,
            "route": client.route.id if client.route is not None else "",
            "latitude": client.member.address.latitude,
            "longitude": client.member.address.longitude,
            "distance": client.member.address.distance,
        }
        return initial

    def get_success_url(self):
        redirect_url = self.request.GET.get("next")
        if redirect_url:
            return redirect_url
        else:
            return reverse_lazy(
                "member:client_information", kwargs={"pk": self.kwargs.get("pk")}
            )

    def form_valid(self, form):
        # This method is called when valid form data has been POSTed.
        # It should return an HttpResponse.
        client = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        self.save(form.cleaned_data, client)
        messages.add_message(
            self.request, messages.SUCCESS, _("The client has been updated")
        )
        return super().form_valid(form)


class ClientUpdateBasicInformation(ClientUpdateInformation):
    form_class = ClientBasicInformation

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({"pk": self.kwargs["pk"], "current_step": "basic_information"})
        context["step_template"] = "client/partials/forms/" "basic_information.html"
        return context

    def save(self, form, client):
        """
        Save the basic information step data.
        """
        client.member.firstname = form["firstname"]
        client.member.lastname = form["lastname"]
        client.member.save()

        client.gender = form["gender"]
        client.birthdate = form["birthdate"]
        client.language = form["language"]
        client.alert = form["alert"]
        client.save()

        # Save contact information
        if client.member.home_phone != form["home_phone"]:
            client.member.add_contact_information(HOME, form["home_phone"], True)
        if client.member.cell_phone != form["cell_phone"]:
            client.member.add_contact_information(CELL, form["cell_phone"], True)
        if client.member.email != form["email"]:
            client.member.add_contact_information(EMAIL, form["email"], True)


class ClientUpdateAddressInformation(ClientUpdateInformation):
    form_class = ClientAddressInformation

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({"current_step": "address_information", "pk": self.kwargs["pk"]})
        context["step_template"] = "client/partials/forms/" "address_information.html"
        return context

    def save(self, form, client):
        """
        Save the basic information step data.
        """
        client.member.address.street = form["street"]
        client.member.address.apartment = form["apartment"]
        client.member.address.city = form["city"]
        client.member.address.postal_code = form["postal_code"]
        client.member.address.latitude = form["latitude"]
        client.member.address.longitude = form["longitude"]
        client.member.address.save()

        client.route = form["route"]
        client.delivery_note = form["delivery_note"]
        client.save()


class ClientUpdatePaymentInformation(ClientUpdateInformation):
    form_class = ClientPaymentInformation

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            {
                "current_step": "payment_information",
                "pk": self.kwargs["pk"],
            }
        )
        context["step_template"] = "client/partials/forms/" "payment_information.html"
        return context

    def get_initial(self):
        initial = super().get_initial()
        client = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        initial.update(
            {
                "firstname": None,
                "lastname": None,
                "street": None,
                "city": None,
                "apartment": None,
                "postal_code": None,
                "member": (
                    f"[{client.billing_member.id}] {client.billing_member.firstname} "
                    f"{client.billing_member.lastname}"
                ),
                "same_as_client": client.member == client.billing_member,
                "facturation": client.rate_type,
                "billing_payment_type": client.billing_payment_type,
            }
        )

        return initial

    def save(self, payment_information, client):
        """
        Save the basic information step data.
        """
        member = client.member
        if payment_information.get("same_as_client"):
            billing_member = member

        else:
            e_b_member = payment_information.get("member")
            if e_b_member:
                e_b_member_id = (
                    e_b_member.split(" ")[0].replace("[", "").replace("]", "")
                )
                billing_member = Member.objects.get(pk=e_b_member_id)
            else:
                billing_address = Address.objects.create(
                    number=payment_information.get("number"),
                    street=payment_information.get("street"),
                    apartment=payment_information.get("apartment"),
                    floor=payment_information.get("floor"),
                    city=payment_information.get("city"),
                    postal_code=payment_information.get("postal_code"),
                )
                billing_address.save()

                billing_member = Member.objects.create(
                    firstname=payment_information.get("firstname"),
                    lastname=payment_information.get("lastname"),
                    address=billing_address,
                )
                billing_member.save()
        client.billing_member = billing_member
        client.rate_type = payment_information.get("facturation")
        client.billing_payment_type = payment_information.get("billing_payment_type")
        client.save()


class ClientUpdateDietaryRestriction(ClientUpdateInformation):
    form_class = ClientRestrictionsInformation

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({"current_step": "dietary_restriction"})
        context.update({"pk": self.kwargs["pk"]})
        context["weekday"] = DAYS_OF_WEEK
        context["meals"] = list(
            filter(
                lambda tup: tup[0] != COMPONENT_GROUP_CHOICES_SIDES,
                COMPONENT_GROUP_CHOICES,
            )
        )
        context["step_template"] = "client/partials/forms/" "dietary_restriction.html"
        return context

    def get_initial(self):
        initial = super().get_initial()
        client = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        initial.update(
            {
                "status": client.status == Client.ACTIVE,
                "delivery_type": client.delivery_type,
                "meals_schedule": client.simple_meals_schedule,
                "restrictions": client.restrictions.all,
                "ingredient_to_avoid": client.ingredients_to_avoid.all,
                "dish_to_avoid": client.components_to_avoid.all,
                "food_preparation": client.food_preparation.all,
            }
        )
        for k, v in (client.meal_default_week or {}).items():
            initial[k] = v
        return initial

    def save(self, form, client):
        """
        Save the basic information step data.
        """
        # Save meals schedule as a Client option
        client.set_simple_meals_schedule(form["meals_schedule"])

        # Save restricted items
        client.restrictions.clear()
        for restricted_item in form["restrictions"]:
            Restriction.objects.create(client=client, restricted_item=restricted_item)

        for food_preparation in client.food_preparation:
            Client_option.objects.filter(
                client=client, option=food_preparation
            ).delete()
        for food_preparation in form["food_preparation"]:
            Client_option.objects.create(client=client, option=food_preparation)

        # Save ingredients to avoid
        client.ingredients_to_avoid.clear()
        for ingredient_to_avoid in form["ingredient_to_avoid"]:
            Client_avoid_ingredient.objects.create(
                client=client, ingredient=ingredient_to_avoid
            )

        # Save components to avoid
        client.components_to_avoid.clear()
        for component_to_avoid in form["dish_to_avoid"]:
            Client_avoid_component.objects.create(
                client=client, component=component_to_avoid
            )

        # Save preferences
        json = {}
        for days, _v in DAYS_OF_WEEK:
            json[f"size_{days}"] = form[f"size_{days}"]

            if json[f"size_{days}"] == "":
                json[f"size_{days}"] = None

            for meal, _Meal in COMPONENT_GROUP_CHOICES:
                if meal is COMPONENT_GROUP_CHOICES_SIDES:
                    continue  # skip "Sides"
                json[f"{meal}_{days}_quantity"] = form[f"{meal}_{days}_quantity"]
        client.delivery_type = form["delivery_type"]
        client.meal_default_week = json
        client.save()


class ClientUpdateRelationshipsInformation(ClientUpdateInformation):
    form_class = UpdateRelationshipFormset
    prefix = "relationships"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({"current_step": "relationships"})
        context.update({"pk": self.kwargs["pk"]})
        context["step_template"] = "client/partials/forms/" "relationships.html"
        return context

    def get_initial(self):
        client = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        initial = {}

        if self.request.method == "GET":
            for i, relationship in enumerate(client.relationship_set.all()):
                contact = relationship.member.member_contact.first()
                if contact:
                    contact_type = contact.type
                    contact_value = contact.value
                else:
                    contact_type = None
                    contact_value = None

                initial[i] = {
                    "firstname": None,
                    "lastname": None,
                    "member": (
                        f"[{relationship.member.id}] {relationship.member.firstname} "
                        f"{relationship.member.lastname}"
                    ),
                    "contact_type": contact_type,
                    "contact_value": contact_value,
                    "nature": relationship.nature,
                    "type": relationship.type,
                    "remark": relationship.remark,
                }
                if relationship.is_referent:
                    initial[i]["referral_date"] = relationship.extra_fields.get(
                        "referral_date"
                    )
                    initial[i]["referral_reason"] = relationship.extra_fields.get(
                        "referral_reason"
                    )

        return initial

    def save(self, relationships, client):
        """
        Save relationships data.
        """
        relationships_posted = []
        for relationship in relationships:
            # Avoid empty forms, at least one relationship form is
            # required by django formset validation. If we have
            # one form good filled and several empty forms, the 'is_valid'
            # method return True
            if relationship:
                r_member = relationship.get("member")
                if r_member:
                    r_member_id = (
                        r_member.split(" ")[0].replace("[", "").replace("]", "")
                    )
                    member = Member.objects.get(pk=r_member_id)
                else:
                    member = Member.objects.create(
                        firstname=relationship.get("firstname"),
                        lastname=relationship.get("lastname"),
                    )
                    if Relationship.REFERENT in relationship.get("type", []):
                        member.work_information = relationship.get("work_information")
                    member.save()

                    # save emergency contact
                    if relationship.get("work_phone"):
                        Contact.objects.create(
                            type=WORK,
                            value=relationship.get("work_phone"),
                            member=member,
                        )
                    elif relationship.get("cell_phone"):
                        Contact.objects.create(
                            type=CELL,
                            value=relationship.get("cell_phone"),
                            member=member,
                        )
                    elif relationship.get("home_phone"):
                        Contact.objects.create(
                            type=HOME,
                            value=relationship.get("home_phone"),
                            member=member,
                        )
                    elif relationship.get("email"):
                        Contact.objects.create(
                            type=EMAIL, value=relationship.get("email"), member=member
                        )

                to_update, _ = Relationship.objects.get_or_create(
                    client=client, member=member
                )
                to_update.nature = relationship.get("nature")
                to_update.type = relationship.get("type")
                extra_fields = {}
                if Relationship.REFERENT in relationship.get("type", []):
                    extra_fields["referral_date"] = relationship.get("referral_date")
                    extra_fields["referral_reason"] = relationship.get(
                        "referral_reason"
                    )
                to_update.extra_fields = extra_fields
                to_update.remark = relationship.get("remark")
                to_update.save()
                relationships_posted.append(to_update)

        Relationship.objects.filter(client=client).exclude(
            pk__in=[c.pk for c in relationships_posted]
        ).delete()


class SearchMembers(LoginRequiredMixin, PermissionRequiredMixin, generic.View):
    permission_required = "sous_chef.read"

    @staticmethod
    def _get_query_args(query):
        """
        Return query arguments built for first_name and
        last_name Member fields
        :return: Q() | Q() or Q() & Q()
        """
        query = query.strip()
        if " " not in query:
            # Is a simple word, check if both fields contains
            # the query value
            return Q(firstname__icontains=query) | Q(lastname__icontains=query)
        else:
            # More than one word, split query to search separately
            # by first and last name
            s_query = query.split(" ")
            return Q(firstname__icontains=s_query[0]) & Q(
                lastname__icontains=" ".join(s_query[1:])
            )

    def get(self, request):
        if request.is_ajax():
            q = request.GET.get("name", "")
            members = Member.objects.filter(self._get_query_args(q))[:20]
            results = []
            for m in members:
                name = "[" + str(m.id) + "] " + m.firstname + " " + m.lastname

                if m.work_information is not None:
                    name += " (" + m.work_information + ")"

                results.append({"title": name})
            data = {"success": True, "results": results}
        else:
            data = {"success": False}

        return JsonResponse(data)


@login_required
def geolocateAddress(request):
    # do something with the your data
    if request.method == "POST":
        lat = request.POST["lat"]
        long = request.POST["long"]

    # just return a JsonResponse
    return JsonResponse({"latitude": lat, "longtitude": long})


class ClientStatusScheduler(
    LoginRequiredMixin,
    PermissionRequiredMixin,
    FormValidAjaxableResponseMixin,
    generic.CreateView,
):
    form_class = ClientScheduledStatusForm
    model = ClientScheduledStatus
    permission_required = "sous_chef.edit"
    template_name = "client/update/status.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["client"] = client = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        context["client_status"] = Client.CLIENT_STATUS
        context["orders"] = client.upcoming_orders.prefetch_related("orders")
        return context

    def get_initial(self):
        client = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        return {
            "client": self.kwargs.get("pk"),
            "status_from": client.status,
            "status_to": self.request.GET.get("status", Client.PAUSED),
        }

    def form_valid(self, form):
        """
        Override the default behavior of saving instance, because we may have
        two objects to save.
        """
        form.save_scheduled_statuses(
            callback_add_message=lambda m: messages.add_message(
                self.request, messages.SUCCESS, m
            )
        )
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse(
            "member:client_information", kwargs={"pk": self.kwargs.get("pk")}
        )


class ClientStatusSchedulerReschedule(
    LoginRequiredMixin,
    PermissionRequiredMixin,
    FormValidAjaxableResponseMixin,
    generic.UpdateView,
):
    form_class = ClientScheduledStatusForm
    model = ClientScheduledStatus
    permission_required = "sous_chef.edit"
    template_name = "client/update/status.html"

    def dispatch(self, *args, **kwargs):
        """
        Fetch related objects at the very beginning of request process.
        """
        self.cs1 = get_object_or_404(
            ClientScheduledStatus, pk=int(self.kwargs.get("scheduled_status_1_pk"))
        )

        if self.kwargs.get("scheduled_status_2_pk"):
            try:
                self.cs2 = ClientScheduledStatus.objects.get(
                    pk=int(self.kwargs.get("scheduled_status_2_pk"))
                )
                if self.cs2.get_pair != self.cs1:
                    self.cs2 = None  # Ignore
            except ClientScheduledStatus.DoesNotExist:
                self.cs2 = None
        else:
            self.cs2 = None
        return super().dispatch(*args, **kwargs)

    def get_object(self, *args, **kwargs):
        return get_object_or_404(Client, pk=self.kwargs.get("pk"))

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["client"] = get_object_or_404(Client, pk=self.kwargs.get("pk"))
        context["client_status"] = Client.CLIENT_STATUS
        return context

    def get_initial(self):
        return {
            "client": self.kwargs.get("pk"),
            "status_from": self.cs1.status_from,
            "status_to": self.cs1.status_to,
            "reason": self.cs1.reason,
            "change_date": self.cs1.change_date,
            "end_date": self.cs2.change_date if self.cs2 else None,
        }

    def form_valid(self, form):
        """
        Replace original ClientScheduledStatus objects.
        """
        # Unbind pairs and delete
        with transaction.atomic():
            self.cs1.delete()
            if self.cs2:
                self.cs2.delete()

        form.save_scheduled_statuses(
            callback_add_message=lambda m: messages.add_message(
                self.request, messages.SUCCESS, m
            )
        )
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse(
            "member:client_information", kwargs={"pk": self.kwargs.get("pk")}
        )


class ClientStatusSchedulerDeleteView(PermissionRequiredMixin, generic.DeleteView):
    model = ClientScheduledStatus
    permission_required = "sous_chef.edit"
    template_name = "client/view/delete_status_confirmation.html"

    def get_success_url(self):
        return reverse("member:client_status", kwargs={"pk": self.object.client.pk})


class DeleteRestriction(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView
):
    model = Restriction
    permission_required = "sous_chef.edit"
    success_url = reverse_lazy("member:list")


class DeleteClientOption(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView
):
    model = Client_option
    permission_required = "sous_chef.edit"
    success_url = reverse_lazy("member:list")


class DeleteIngredientToAvoid(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView
):
    model = Client_avoid_ingredient
    permission_required = "sous_chef.edit"
    success_url = reverse_lazy("member:list")


class DeleteComponentToAvoid(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DeleteView
):
    model = Client_avoid_component
    permission_required = "sous_chef.edit"
    success_url = reverse_lazy("member:list")


class RouteListView(LoginRequiredMixin, PermissionRequiredMixin, generic.ListView):
    # Display the list of routes
    context_object_name = "routes"
    model = Route
    queryset = Route.objects.all().annotate(
        client_count=Sum(
            Case(
                When(
                    client__status__in=[Client.PENDING, Client.ACTIVE, Client.PAUSED],
                    then=1,
                ),
                default=0,
                output_field=IntegerField(),
            )
        )
    )
    permission_required = "sous_chef.read"
    template_name = "route/list.html"


def get_clients_on_route(route):
    """
    Helper function, intended to generate the context variable of
    Route-related views.

    Returns a list of client instances with an extra attribute
    `has_been_configured`, ordered by configured then unconfigured.
    """
    clients = Client.objects.filter(
        route=route, status__in=[Client.PENDING, Client.ACTIVE, Client.PAUSED]
    ).select_related("member", "member__address")
    clients_dict = {client.pk: client for client in clients}
    clients_on_route = []
    for client_pk in route.client_id_sequence or []:
        if client_pk in clients_dict:
            client = clients_dict[client_pk]
            client.has_been_configured = True
            clients_on_route.append(client)
            del clients_dict[client_pk]
        else:
            # This client has been deleted after configuring the route.
            # We don't include it in the sequence. Thus, when the user updates
            # the default route sequence the next time, everything will be OK.
            pass

    for _client_pk, client in clients_dict.items():
        # Remaining clients
        client.has_been_configured = False
        clients_on_route.append(client)

    return clients_on_route


def get_clients_on_delivery_history(delivery_history, func_add_warning_message=None):
    """
    Helper function, intended to generate the context variable of
    DeliveryHistory-related views.

    Returns a list of client instances with extra attributes
    `has_been_configured` and `order_of_the_day`, ordered by configured
    then unconfigured.

    `order_of_the_day` could be none if we can't find the order and the
    client is on the sequence. We assume that the order was deleted.

    `func_add_warning_message` accepts one single string-like parameter.
    It is called when we find a data integrity problem. That is when
    there's a client that doesn't exist or we can't find his order
    on that day any more.

    See also: member.tests.RouteDeliveryHistoryDetailViewTestCase
    """
    orders = Order.objects.get_shippable_orders_by_route(
        delivery_history.route.pk,
        delivery_history.date,
        exclude_non_geolocalized=True,
    ).select_related("client", "client__member", "client__member__address")
    clients = []
    clients_dict = {}
    for order in orders:
        c = order.client
        c.order_of_the_day = order
        clients.append(c)
        clients_dict[c.pk] = c

    clients_on_delivery_history = []

    # First, check the clients on `id_sequence` and see if we put them in the
    # display sequence.
    for client_pk in delivery_history.client_id_sequence or []:
        if client_pk in clients_dict:
            client = clients_dict[client_pk]
            client.has_been_configured = True
            clients_on_delivery_history.append(client)
            del clients_dict[client_pk]
        else:
            # Oops, a data integrity error. Let user know.
            if func_add_warning_message:
                try:
                    client = Client.objects.select_related("member").get(
                        pk=int(client_pk)
                    )
                    func_add_warning_message(
                        mark_safe(
                            _(
                                "The client <a href={url}>{firstname} {lastname}"
                                "</a> is found on this delivery sequence but is no "
                                "longer valid for this delivery."
                            ).format(
                                firstname=client.member.firstname,
                                lastname=client.member.lastname,
                                url=reverse(
                                    "member:client_information", args=(client.pk,)
                                ),
                            )
                        )
                    )
                except (Client.DoesNotExist, ValueError, TypeError):
                    func_add_warning_message(
                        _(
                            "The delivery sequence contains a client with ID"
                            "{client_id} that no longer exists in the "
                            "database."
                        ).format(client_id=client_pk)
                    )

    # Then, check if there are clients that exist on default route sequence.
    # They are considered as "by default".
    for cpk in delivery_history.route.client_id_sequence or []:
        if cpk in clients_dict:
            client = clients_dict[cpk]
            client.has_been_configured = False
            clients_on_delivery_history.append(client)
            del clients_dict[cpk]

    # Finally, check the clients that had delivery on that day but were not
    # configured on the delivery sequence. Append them to the display sequence.
    for _client_pk, client in clients_dict.items():
        client.has_been_configured = False
        clients_on_delivery_history.append(client)
    return clients_on_delivery_history


class RouteDetailView(LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView):
    model = Route
    queryset = Route.objects.prefetch_related(
        Prefetch(
            "delivery_histories", queryset=DeliveryHistory.objects.order_by("-date")
        )
    )
    permission_required = "sous_chef.read"
    template_name = "route/detail.html"

    def get_context_data(self, **kwargs):
        """
        Get detailed information on route clients and out-of-route clients
        that are to be rendered on the template.
        """
        context = super().get_context_data(**kwargs)
        route = context["route"]
        context["clients_on_route"] = get_clients_on_route(route)
        return context


class RouteEditView(
    LoginRequiredMixin, PermissionRequiredMixin, generic.edit.UpdateView
):
    model = Route
    fields = ("name", "description", "vehicle", "client_id_sequence")
    permission_required = "sous_chef.edit"
    template_name = "route/edit.html"

    def get_context_data(self, **kwargs):
        """
        Get detailed information on route clients and out-of-route clients
        that are to be rendered on the template.
        """
        context = super().get_context_data(**kwargs)
        route = context["route"]
        context["clients_on_route"] = get_clients_on_route(route)
        return context

    def get_success_url(self):
        return reverse_lazy("member:route_detail", args=[self.object.pk])

    def form_valid(self, form):
        response = super().form_valid(form)
        messages.add_message(
            self.request, messages.SUCCESS, _("This route has been updated.")
        )
        return response


@login_required
def get_minimised_euclidean_distances_route_sequence(request, pk):
    """
    Return the sequence of clients on the given route that minimises
    euclidean distances, as a JSON list of client IDs.
    """
    route = get_object_or_404(Route, pk=pk)
    clients_on_route = get_clients_on_route(route)
    waypoints = list(
        map(
            lambda c: {
                "id": c.pk,
                "latitude": c.member.address.latitude,
                "longitude": c.member.address.longitude,
            },
            clients_on_route,
        )
    )
    optimised_waypoints = calculateRoutePointsEuclidean(waypoints)
    return JsonResponse(list(map(lambda w: w["id"], optimised_waypoints)), safe=False)


class DeliveryHistoryDetailView(
    LoginRequiredMixin, PermissionRequiredMixin, generic.DetailView
):
    model = DeliveryHistory
    permission_required = "sous_chef.read"
    template_name = "route/delivery_history_detail.html"

    def get_object(self, *args, **kwargs):
        return DeliveryHistory.objects.select_related("route").get(
            route=self.kwargs.get("route_pk"), date=self.kwargs.get("date")
        )

    def get_context_data(self, **kwargs):
        """
        Get detailed information on route clients and out-of-route clients
        that are to be rendered on the template.
        """
        context = super().get_context_data(**kwargs)
        delivery_history = self.object
        context["delivery_history"] = delivery_history
        context["clients_on_delivery_history"] = get_clients_on_delivery_history(
            delivery_history,
            func_add_warning_message=lambda msg: messages.add_message(
                self.request, messages.WARNING, msg
            ),
        )
        return context



# Generated by Django 1.11.29 on 2021-04-02 19:15

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0016_auto_20180704_1613"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order_item",
            name="total_quantity",
            field=models.IntegerField(default=0, verbose_name="total quantity"),
            preserve_default=False,
        ),
    ]



from django.contrib.auth.models import User
from django.db import models
from django.db.models import Q
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django_filters import (
    CharFilter,
    ChoiceFilter,
    DateFromToRangeFilter,
    FilterSet,
)

# Create your models here.


class NoteManager(models.Manager):
    pass


class UnreadNoteManager(NoteManager):
    def get_queryset(self):
        return super().get_queryset().filter(is_read=0)


class NotePriority(models.Model):
    name = models.CharField(max_length=150, verbose_name=_("Name"))

    class Meta:
        verbose_name_plural = _("Note priorities")
        ordering = ("name",)

    def __str__(self):
        return "%s" % self.name


class NoteCategory(models.Model):
    name = models.CharField(max_length=150, verbose_name=_("Name"))

    class Meta:
        verbose_name_plural = _("Note categories")
        ordering = ("name",)

    def __str__(self):
        return "%s" % self.name


class Note(models.Model):
    class Meta:
        verbose_name_plural = _("Notes")
        ordering = ("-date_modified", "note")

    note = models.TextField(verbose_name=_("Note"))

    author = models.ForeignKey(
        User, verbose_name=_("Author"), null=True, on_delete=models.SET_NULL
    )

    date_created = models.DateTimeField(
        verbose_name=_("Date Created"),
        default=timezone.now,
    )

    date_modified = models.DateTimeField(
        verbose_name=_("Date Modified"),
        default=timezone.now,
    )

    is_read = models.BooleanField(verbose_name=_("Is read"), default=False)

    client = models.ForeignKey(
        "member.Client",
        verbose_name=_("Client"),
        related_name="client_notes",
        on_delete=models.CASCADE,
    )

    priority = models.ForeignKey(
        NotePriority,
        verbose_name=_("Priority"),
        related_name="notes",
        null=True,
        on_delete=models.SET_NULL,
    )

    category = models.ForeignKey(
        NoteCategory,
        verbose_name=_("Category"),
        related_name="notes",
        null=True,
        on_delete=models.SET_NULL,
    )

    is_deleted = models.BooleanField(verbose_name=_("Is deleted"), default=False)

    objects = NoteManager()

    unread = UnreadNoteManager()

    def __str__(self):
        return self.note

    def mark_as_read(self):
        """Mark a note as read."""
        if not self.is_read:
            self.is_read = True
            self.save()

    def mark_as_unread(self):
        """Mark a note as unread."""
        if self.is_read:
            self.is_read = False
            self.save()

    def delete(self):
        """Mark a note as being deleted."""
        self.is_deleted = True
        self.save()


class NoteFilter(FilterSet):
    IS_READ_CHOICES = (
        ("", "All"),
        ("1", "Yes"),
        ("0", "No"),
    )

    NOTE_STATUS_UNREAD = IS_READ_CHOICES[2][0]

    is_read = ChoiceFilter(
        choices=IS_READ_CHOICES,
    )

    name = CharFilter(method="filter_search", label=_("Search by name"))

    date_modified = DateFromToRangeFilter(lookup_expr="contains")

    class Meta:
        model = Note
        fields = [
            "priority",
            "is_read",
            "date_modified",
            "category",
        ]

    def filter_search(self, queryset, field_name, value):
        if not value:
            return queryset

        names = value.split(" ")

        name_contains = Q()

        for name in names:
            firstname_contains = Q(client__member__firstname__icontains=name)

            name_contains |= firstname_contains

            lastname_contains = Q(client__member__lastname__icontains=name)
            name_contains |= lastname_contains

        return queryset.filter(name_contains)



# Generated by Django 1.9.6 on 2016-06-14 17:36

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0001_fix161f"),
    ]

    operations = [
        migrations.AlterField(
            model_name="order_item",
            name="order",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="orders",
                to="order.Order",
                verbose_name="order",
            ),
        ),
    ]



COMPONENT_SYSTEM_DEFAULT = {
    "main_dish": 1,
    "size": "R",
    "dessert": 1,
    "diabetic": 0,
    "fruit_salad": 0,
    "green_salad": 0,
    "pudding": 0,
    "compote": 0,
    "sides": 0,
}



from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class Notification(models.Model):
    class Meta:
        verbose_name_plural = _("notifications")

    description = models.TextField(verbose_name=_("description"))

    member = models.ForeignKey(
        "member.Member", verbose_name=_("member"), on_delete=models.CASCADE
    )

    date = models.DateField(auto_now=False, auto_now_add=False, default=timezone.now)

    def __str__(self):
        return self.description



from datetime import (
    date,
    datetime,
)

from django.core.management.base import BaseCommand

from souschef.member.models import ClientScheduledStatus


class Command(BaseCommand):
    help = "Process scheduled status changes, \
        queued in «member_ClientScheduledStatus» table."

    def handle(self, *args, **options):
        # List all change not processed, and older or equal to now
        changes = ClientScheduledStatus.objects.filter(
            operation_status=ClientScheduledStatus.TOBEPROCESSED
        ).filter(change_date__lte=date.today())

        # For each change to be processed,
        for scheduled_change in changes:
            if scheduled_change.process():
                suc_msg = (
                    f": client «{scheduled_change.client.member}» status updated "
                    f"from {scheduled_change.get_status_from_display()} to "
                    f"{scheduled_change.get_status_to_display()}."
                )
                self.stdout.write(self.style.SUCCESS(str(datetime.now()) + suc_msg))
            # If not, mark change as processed with error
            else:
                err_msg = (
                    f": client «{scheduled_change.client.member}» status not updated."
                )
                err_msg += (
                    " Current status is "
                    f"«{scheduled_change.client.get_status_display()}», should be "
                    f"«{scheduled_change.get_status_from_display()}»."
                )
                self.stdout.write(self.style.ERROR(str(datetime.now()) + err_msg))



# Generated by Django 1.9.5 on 2016-06-05 21:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meal", "0001_fix161f"),
    ]

    operations = [
        migrations.AddField(
            model_name="ingredient",
            name="ingredient_group",
            field=models.CharField(
                choices=[
                    ("meat", "Meat"),
                    ("dairy", "Dairy"),
                    ("fish", "Fish"),
                    ("seafood", "Seafood"),
                    ("veggies and fruits", "Veggies and fruits"),
                    ("legumineuse", "Legumineuse"),
                    ("grains", "Grains"),
                    ("fresh herbs", "Fresh herbs"),
                    ("spices", "Spices"),
                    ("dry and canned goods", "Dry and canned goods"),
                    ("oils and sauces", "Oils and sauces"),
                ],
                default="",
                max_length=100,
                verbose_name="ingredient group",
            ),
            preserve_default=False,
        ),
    ]



from django.forms import formset_factory

from .forms import ClientRelationshipInformation

CreateRelationshipFormset = formset_factory(
    ClientRelationshipInformation, min_num=0, extra=0
)
UpdateRelationshipFormset = formset_factory(
    ClientRelationshipInformation, min_num=0, extra=0
)



from django.apps import apps
from django.conf import settings
from django.contrib.auth.models import User
from django.core.management import call_command
from django.db import connection
from django.db.migrations.executor import MigrationExecutor
from django.test import TransactionTestCase
from django.utils.http import urlquote

METHODS = ("GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONS")


class TestMixin:
    def assertRedirectsWithAllMethods(self, url, methods=METHODS, **kwargs):
        """
        Test a URL with all HTTP methods, as not logged-in guests.
        """
        self.client.logout()
        for method in methods:
            response = getattr(self.client, method.lower())(url)
            self.assertRedirects(
                response,
                urlquote(settings.LOGIN_URL) + "?next=" + urlquote(url),
                status_code=302,
                msg_prefix=f"{method} {url} ",
                **kwargs,
            )

    def force_login(self, role="admin"):
        """
        Create administrator and force login for testing purposes.

        Possible to extend `role` in the future.
        """
        if User.objects.filter(is_superuser=True).exists():
            admin = User.objects.filter(is_superuser=True).first()
        else:
            admin = User.objects.create_superuser(
                username="testadmin", email="testadmin@example.com", password="test1234"
            )
        self.client.force_login(
            admin,
            "django.contrib.auth.backends.ModelBackend",
        )


class TestMigrations(TransactionTestCase):
    """
    https://www.caktusgroup.com/blog/2016/02/02/writing-unit-tests-django-migrations/
    https://gist.github.com/blueyed/4fb0a807104551f103e6
    """

    @property
    def app(self):
        name = apps.get_containing_app_config(type(self).__module__).name

        # We want the name of the app and not all the module path
        # sous-chef.member -> member
        #
        # Actually all apps use the name of the folder, so we can just get
        # the second part of a split
        # todo: enhance this logic to use the content of apps.py
        short_name = name.split(".")[1]

        return short_name

    migrate_from = None
    migrate_to = None

    def setUp(self):
        assert self.migrate_from and self.migrate_to, (
            f"TestCase '{type(self).__name__}' must define migrate_from and "
            "migrate_to properties"
        )
        self.migrate_from = [(self.app, self.migrate_from)]
        self.migrate_to = [(self.app, self.migrate_to)]
        executor = MigrationExecutor(connection)
        old_apps = executor.loader.project_state(self.migrate_from).apps

        # Reverse to the original migration
        executor.migrate(self.migrate_from)

        self.setUpBeforeMigration(old_apps)

        # Run the migration to test
        executor.loader.build_graph()
        executor.migrate(self.migrate_to)

        self.apps = executor.loader.project_state(self.migrate_to).apps

    def setUpBeforeMigration(self, apps):
        raise NotImplementedError(
            f"TestCase '{type(self).__name__}' must define the method "
            "setUpBeforeMigration(self, apps)"
        )

    def tearDown(self):
        """
        In the end, ensure that the tested app has its latest migration.
        """
        call_command("migrate", self.app, verbosity=0)



import collections
from datetime import (
    date,
    datetime,
)

from annoying.fields import JSONField
from django.db import models
from django.db.models import (
    Prefetch,
    Q,
)
from django.utils.translation import ugettext_lazy as _
from django_filters import (
    CharFilter,
    FilterSet,
)

from souschef.order.models import (
    Order,
    Order_item,
)


class BillingManager(models.Manager):
    def billing_create_new(self, year, month):
        """
        Create a new billing for the given period.
        A period is a month.
        """
        # Get all billable orders for the given period
        billable_orders = (
            Order.objects.get_billable_orders(year, month)
            .select_related("client__member")
            .only("client__member__firstname", "client__member__lastname")
            .prefetch_related(
                Prefetch(
                    "orders",
                    queryset=Order_item.objects.all().only(
                        "order__id", "price", "billable_flag"
                    ),
                )
            )
        )

        total_amount = calculate_amount_total(billable_orders)

        # Create the Billing object
        billing = Billing.objects.create(
            total_amount=total_amount,
            billing_month=month,
            billing_year=year,
            created=datetime.today(),
            detail={},
        )

        # Attach the orders
        billing.orders.add(*billable_orders)

        return billing

    def billing_get_period(self, year, month):
        """
        Check if a billing exists for a given period.
        Return None otherwise.
        """
        billing = Billing.objects.filter(
            billing_year=year,
            billing_month=month,
        )

        if billing.count() == 0:
            return None
        else:
            return billing


class Billing(models.Model):  # noqa: DJ008
    class Meta:
        ordering = ["-billing_year", "-billing_month"]

    total_amount = models.DecimalField(
        verbose_name=_("total_amount"), max_digits=8, decimal_places=2
    )

    # Month start with january is 1
    billing_month = models.IntegerField()

    billing_year = models.IntegerField()

    created = models.DateTimeField(verbose_name=None, auto_now=True)

    detail = JSONField()

    orders = models.ManyToManyField(Order)

    objects = BillingManager()

    @property
    def billing_period(self):
        """
        Return a readable format for the billing period.
        """
        period = date(self.billing_year, self.billing_month, 1)
        return period

    @property
    def summary(self):
        """
        Return a summary of every client.
        Format: dictionary {client: info}
        """
        # collect orders by clients
        kvpairs = map(lambda o: (o.client, o), self.orders.all())
        d = collections.defaultdict(list)
        for k, v in kvpairs:
            d[k].append(v)
        result = {}
        for client, orders in d.items():
            result[client] = {
                "total_orders": len(orders),
                "total_main_dishes": {"R": 0, "L": 0},  # to be counted
                "total_billable_sides": 0,  # to be counted
                "total_amount": sum(map(lambda o: o.price, orders)),
            }
            for o in orders:
                for o_item in o.orders.all():
                    if o_item.component_group == "main_dish":
                        if o_item.size == "R":
                            result[client]["total_main_dishes"][
                                "R"
                            ] += o_item.total_quantity
                        elif o_item.size == "L":
                            result[client]["total_main_dishes"][
                                "L"
                            ] += o_item.total_quantity
                    else:
                        if o_item.billable_flag is True:
                            result[client][
                                "total_billable_sides"
                            ] += o_item.total_quantity
        return result


class BillingFilter(FilterSet):
    name = CharFilter(method="filter_search", label=_("Search by name"))

    date = CharFilter(method="filter_period")

    class Meta:
        model = Billing
        fields = "__all__"

    def filter_search(self, queryset, field_name, value):
        if not value:
            return queryset

        name_contains = Q()
        names = value.split(" ")

        for name in names:
            firstname_contains = Q(client__member__firstname__icontains=name)

            lastname_contains = Q(client__member__lastname__icontains=name)

            name_contains |= firstname_contains | lastname_contains

        return queryset.filter(name_contains)

    def filter_period(self, queryset, field_name, value):
        if not value:
            return queryset

        year, month = value.split("-")
        return queryset.filter(billing_year=year, billing_month=month)


# get the total amount from a list of orders
def calculate_amount_total(orders):
    total = 0
    for order in orders:
        total += order.price
    return total



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from souschef.delivery.views import (
    CreateDelivery,
    DeliveryRouteSheet,
    EditDeliveryRoute,
    KitchenCount,
    MealInformation,
    RefreshOrderView,
    ReviewOrders,
    RoutesInformation,
)

app_name = "delivery"

urlpatterns = [
    url(_(r"^order/$"), ReviewOrders.as_view(), name="order"),
    url(_(r"^meal/$"), MealInformation.as_view(), name="meal"),
    url(_(r"^meal/(?P<id>\d+)/$"), MealInformation.as_view(), name="meal_id"),
    url(_(r"^routes/$"), RoutesInformation.as_view(), name="routes"),
    url(
        _(r"^route/(?P<pk>\d+)/$"),
        EditDeliveryRoute.as_view(),
        name="edit_delivery_route",
    ),
    url(
        _(r"^route/(?P<pk>\d+)/create/$"),
        CreateDelivery.as_view(),
        name="create_delivery",
    ),
    url(_(r"^kitchen_count/$"), KitchenCount.as_view(), name="kitchen_count"),
    url(
        _(r"^route_sheet/(?P<pk>\d+)/$"),
        DeliveryRouteSheet.as_view(),
        name="route_sheet",
    ),
    url(_(r"^refresh_orders/$"), RefreshOrderView.as_view(), name="refresh_orders"),
]



# Generated by Django 1.9.6 on 2016-06-30 15:34

import annoying.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("member", "0005_fix004a"),
    ]

    operations = [
        migrations.AddField(
            model_name="client",
            name="meal_default_week",
            field=annoying.fields.JSONField(blank=True, null=True),
        ),
    ]



"""
Django settings for sous-chef project.

Generated by 'django-admin startproject' using Django 1.9.2.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

import os
import subprocess

from django.urls import reverse_lazy

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get(
    "SOUSCHEF_DJANGO_SECRET_KEY", "15ine$#^qas4_h2u7yk&lxat*&g*b8+)@wp$2x@vi2#v9)i2#u"
)  # For dev purposes

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.environ.get("SOUSCHEF_ENVIRONMENT_NAME") == "DEV"

# This IP may change for different computers and should be the
# request.META.get('REMOTE_ADDR') for your local computer.
# Don't run with this in production and don't
# commit any changes to this INTERNAL_IPS settings.
# WE NEED THIS IN ORDER TO USE 'debug' IN TEMPLATES!!!
INTERNAL_IPS = (
    os.environ.get(
        "SOUSCHEF_DJANGO_INTERNAL_IPS",
        # Default value is an example: use space-separated string in env
        "172.19.0.1  172.22.0.1",
    )
    .strip()
    .split()
)

ALLOWED_HOSTS = (
    os.environ.get(
        "SOUSCHEF_DJANGO_ALLOWED_HOSTS",
        # Default value is an example: use space-separated string in env
        "localhost souschef.example.com test.souschef.example.com",
    )
    .strip()
    .split()
)


# Application definition

INSTALLED_APPS = [
    # Django apps
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # Third party apps
    "annoying",
    "avatar",
    "django_crontab",
    "django_filters",
    "formtools",
    "leaflet",
    "rules.apps.AutodiscoverRulesConfig",
    # Sous-chef apps
    "souschef.sous_chef",
    "souschef.billing",
    "souschef.datamigration",
    "souschef.delivery",
    "souschef.meal",
    "souschef.member",
    "souschef.order",
    "souschef.notification",
    "souschef.page",
    "souschef.note",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.auth.middleware.SessionAuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.locale.LocaleMiddleware",
]

ROOT_URLCONF = "souschef.sous_chef.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [BASE_DIR + "/sous_chef/templates/"],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.i18n",
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "souschef.sous_chef.context_processors.total",
            ],
        },
    },
]

WSGI_APPLICATION = "souschef.sous_chef.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.mysql",
        "NAME": os.environ.get("SOUSCHEF_DJANGO_DB_NAME", "feast"),
        "USER": os.environ.get("SOUSCHEF_DJANGO_DB_USER", "root"),
        "PASSWORD": os.environ.get("SOUSCHEF_DJANGO_DB_PASSWORD", "123456"),
        "HOST": os.environ.get("SOUSCHEF_DJANGO_DB_HOST", "db"),
        "port": os.environ.get("SOUSCHEF_DJANGO_DB_PORT", "3306"),
    }
}


# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation."
        "UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation." "MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation." "CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation." "NumericPasswordValidator",
    },
]

LOGIN_URL = reverse_lazy("page:login")
LOGIN_REDIRECT_URL = reverse_lazy("page:home")

AUTHENTICATION_BACKENDS = (
    "rules.permissions.ObjectPermissionBackend",
    "django.contrib.auth.backends.ModelBackend",
)


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = "en"
USE_I18N = True
USE_L10N = True

# Store datetimes as UTC in database
USE_TZ = True

# Use this timezone when displaying datetimes
TIME_ZONE = "America/Montreal"

# List of supported languages
LANGUAGES = (
    ("fr", "Français"),
    ("en", "English"),
)

LOCALE_PATHS = tuple(
    map(
        lambda path: os.path.join(BASE_DIR, path),
        (
            "meal/locale/",
            "member/locale/",
            "notification/locale/",
            "order/locale/",
            "page/locale/",
            "delivery/locale/",
            "billing/locale",
            "note/locale",
        ),
    )
)

FORMAT_MODULE_PATH = ("souschef.sous_chef.formats",)

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/
STATIC_ROOT = os.path.join(BASE_DIR, "static")

STATICFILES_DIRS = (BASE_DIR + "/sous_chef/assets/",)
STATIC_URL = "/static/"

GENERATED_DOCS_DIR = os.environ.get(
    "SOUSCHEF_GENERATED_DOCS_DIR", "/var/local/souschef"
)

CRONJOBS = [("0 0 * * *", "souschef.pycrons.cleaning.clean_old_pdf_files")]

if DEBUG:
    # When using the development server, serve files directly from /media/
    # https://docs.djangoproject.com/en/1.11/howto/static-files/#serving-files-uploaded-by-a-user-during-development
    MEDIA_ROOT = os.path.join(BASE_DIR, "media")
    MEDIA_URL = "/media/"
    GENERATED_DOCS_DIR = os.environ.get("SOUSCHEF_GENERATED_DOCS_DIR", BASE_DIR)
else:
    # In non development mode, serve files from /static/ using nginx as
    # dedicated server
    # https://docs.djangoproject.com/en/1.11/howto/static-files/deployment/#serving-static-files-from-a-dedicated-server
    MEDIA_ROOT = os.path.join(STATIC_ROOT, "media")
    MEDIA_URL = "/static/media/"

# Avatar files
AVATAR_PROVIDERS = (
    "avatar.providers.PrimaryAvatarProvider",
    "avatar.providers.GravatarAvatarProvider",
    "avatar.providers.DefaultAvatarProvider",
)

MEAL_LABELS_FILE = os.path.join(GENERATED_DOCS_DIR, "meal_labels.pdf")
KITCHEN_COUNT_FILE = os.path.join(GENERATED_DOCS_DIR, "kitchen_count.pdf")
ROUTE_SHEETS_FILE = os.path.join(GENERATED_DOCS_DIR, "route_sheets.pdf")

# Displayable information
SOUSCHEF_ENVIRONMENT_NAME = os.environ.get("SOUSCHEF_ENVIRONMENT_NAME") or ""

try:
    GIT_HEAD = subprocess.check_output(["git", "rev-parse", "HEAD"])
    GIT_TAG = subprocess.check_output(["git", "describe", "--tags"])
except Exception:
    GIT_HEAD = None
    GIT_TAG = None



# Generated by Django 1.10 on 2016-11-22 05:15

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("billing", "0002_auto_20161122_0458"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="billing",
            options={"ordering": ["billing_year", "-billing_month"]},
        ),
    ]



from django import forms

from souschef.member.models import Client
from souschef.note.models import Note


class NoteForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["client"].queryset = (
            Client.objects.all()
            .select_related("member")
            .only("member__firstname", "member__lastname")
        )

    class Meta:
        model = Note
        fields = ["note", "client", "priority", "category"]

        widgets = {
            "note": forms.Textarea(attrs={"rows": 7}),
            "client": forms.Select(attrs={"class": "ui search dropdown"}),
            "priority": forms.Select(attrs={"class": "ui dropdown"}),
            "category": forms.Select(attrs={"class": "ui dropdown"}),
        }


class NoteEditForm(forms.ModelForm):
    class Meta:
        model = Note
        fields = ["note", "priority", "category"]

        widgets = {
            "note": forms.Textarea(attrs={"rows": 7}),
            "priority": forms.Select(attrs={"class": "ui dropdown"}),
            "category": forms.Select(attrs={"class": "ui dropdown"}),
        }



from .settings import *  # noqa

SECRET_KEY = "SecretKeyForUseOnTravis"

DEBUG = False
TEMPLATE_DEBUG = False

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}
